import xbmc
import xbmcgui
import xbmcaddon
import xbmcvfs
import json
import random
import sys
import urllib

ADDON = xbmcaddon.Addon()
CWD = ADDON.getAddonInfo('path').decode('utf-8')
#CWD = ADDON.getAddonInfo('path') # for kodi 19

def list_programs(cinematic_path):
    dirs, files = xbmcvfs.listdir(cinematic_path)
    programs = {}
    for filename in files:
        if filename.endswith('.json'):
            filehandle = xbmcvfs.File(cinematic_path + filename)
            program_json = filehandle.read()
            filehandle.close()
            program_data = json.loads(program_json)
            programs[program_data['name']] = filename
    return programs

def show_dialog(programs):
    entries = programs.keys()
    dialog = xbmcgui.Dialog()
    ret = dialog.select('Cinematic: Select a program', entries)
    del dialog
    return programs[entries[ret]]

def files_from_dir(count, location):
    dirs, files = xbmcvfs.listdir(location)
    files = random.sample(files, count)
    for i in range(len(files)):
        files[i] = location + files[i]
    return files

def get_recommendations(apikey, movieid, language, choice):
    print("getting %s for %d" % (choice, movieid))
    baseurl = 'https://api.themoviedb.org/3/'
    url = baseurl + 'movie/%d/%s?api_key=%s&language=%s'
    url = url % (movieid, choice, apikey, language)
    json_url = urllib.urlopen(url)
    data = json.loads(json_url.read())
    results = []
    for result in data['results']:
        results.append({'title': result['title'], 'movieid': result['id']})
    return results

def get_movie_trailers(apikey, movieid, language):
    print("getting trailers for %d" % movieid)
    baseurl = 'https://api.themoviedb.org/3/'
    url = baseurl + 'movie/%d?api_key=%s&language=%s&append_to_response=videos'
    url = url % (movieid, apikey, language)
    json_url = urllib.urlopen(url)
    data = json.loads(json_url.read())
    results = []
    for result in data['videos']['results']:
        if result['site'] == 'YouTube':
            location = 'plugin://plugin.video.youtube/play/?video_id=%s' % result['key']
        else:
            next
        results.append({'title': result['name'], 'type': result['type'], 'location': location})
    return results

def get_trailers(apikey, recommendations, language, cliptype, count):
    results = []
    for recommendation in recommendations[:10]:
        all_trailers = get_movie_trailers(apikey, recommendation['movieid'], language)
        trailers = []
        for trailer in all_trailers:
            if trailer['type'] == cliptype:
                trailers.append(trailer)
        if len(trailers) > 0:
            random.shuffle(trailers)
            trailer = trailers[0]
            results.append(trailer)
        if len(results) == count:
            break
    return results

def conduct_program(program_file, feature):
    filehandle = xbmcvfs.File(program_file)
    program_json = filehandle.read()
    filehandle.close()
    program_data = json.loads(program_json)
    program = []
    for item in program_data['items']:
        settings = item['settings']
        if settings['source'] == 'file':
            entry = {'type': 'video', 'data': settings['location']}
            program.append(entry)
        elif settings['source'] == 'dir':
            for location in files_from_dir(settings['count'], settings['location']):
                entry = {'type': 'video', 'data': location}
                program.append(entry)
        elif settings['source'] == 'tmdbtrailer':
            apikey = settings['apikey']
            tmdbid = int(feature['tmdbid'])
            language = settings['language']
            choice = settings['choice']
            trailertype = settings['type']
            count = settings['count']
            movies = get_recommendations(apikey, tmdbid, language, choice)
            random.shuffle(movies)
            trailers = get_trailers(apikey, movies, language, trailertype, count)
            for trailer in trailers:
                entry = {'type': 'video', 'data': trailer['location']}
                program.append(entry)
        elif settings['source'] == 'feature':
            entry = {'type': 'video', 'data': feature['file']}
            program.append(entry)
    return program

def get_feature(movieid):
    query = '{"jsonrpc": "2.0", "method": "VideoLibrary.GetMovieDetails", "params": {"movieid": %s, "properties": ["file", "mpaa", "uniqueid"]}, "id": "1"}' % movieid
    json_response = xbmc.executeJSONRPC(query)
    response = json.loads(json_response)
    feature = {
       'label': response['result']['moviedetails']['label'],
       'file': response['result']['moviedetails']['file'],
       'mpaa': response['result']['moviedetails']['mpaa'],
       'tmdbid': response['result']['moviedetails']['uniqueid']['tmdb']
    }
    return feature

if __name__ == '__main__':
    for arg in sys.argv[1:]:
        (name, value) = arg.split('=')
        if name == 'dbid':
            movieid = int(value)

    cinematic_path = ADDON.getSettingString('cinematic_path')
    feature = get_feature(movieid)
    print(feature)
    programs = list_programs(cinematic_path)
    program_file = cinematic_path + show_dialog(programs)
    program = conduct_program(program_file, feature)

    print('=== playlist')
    for entry in program:
        print(" * [%s] -- %s" % (entry['type'], entry['data']))
